==> sinabs-0.0.0.tar.gz: sinabs-0.0.0/tests/test_activations.py <==
import pytest
import torch
from sinabs.activation import (
    ActivationFunction,
    ALIFActivationFunction,
    MembraneSubtract,
    MembraneReset,
    MultiSpike,
    SingleSpike,
)


@pytest.mark.parametrize(
    "act_fn, output, v_mem",
    [
        (
            ActivationFunction(spike_fn=SingleSpike, reset_fn=MembraneSubtract()),
            torch.tensor([1.0, 0.0]),
            torch.tensor([1.5, 0.3]),
        ),
        (
            ActivationFunction(spike_fn=SingleSpike, reset_fn=MembraneReset()),
            torch.tensor([1.0, 0.0]),
            torch.tensor([0.0, 0.3]),
        ),
        (
            ActivationFunction(spike_fn=MultiSpike, reset_fn=MembraneSubtract()),
            torch.tensor([2.0, 0.0]),
            torch.tensor([0.5, 0.3]),
        ),
        (
            ActivationFunction(spike_fn=MultiSpike, reset_fn=MembraneReset()),
            torch.tensor([2.0, 0.0]),
            torch.tensor([0.0, 0.3]),
        ),
    ],
)
def test_activation_functions(act_fn, output, v_mem):
    state = {"v_mem": torch.tensor([2.5, 0.3], requires_grad=True)}

    spikes, new_state = act_fn(state)

    assert torch.allclose(spikes, output)
    assert torch.allclose(new_state["v_mem"], v_mem)

    loss = torch.nn.functional.mse_loss(spikes, torch.ones_like(spikes))
    loss.backward()

    assert not (state["v_mem"].grad == 0).all()
    assert not torch.isnan(state["v_mem"].grad).any()
    assert not torch.isinf(state["v_mem"].grad).any()


@pytest.mark.parametrize(
    "act_fn, output, v_mem",
    [
        (
            ALIFActivationFunction(spike_fn=SingleSpike, reset_fn=MembraneSubtract()),
            torch.tensor([1.0, 0.0]),
            torch.tensor([0.4, 1.5]),
        ),
        (
            ALIFActivationFunction(spike_fn=SingleSpike, reset_fn=MembraneReset()),
            torch.tensor([1.0, 0.0]),
            torch.tensor([0.0, 1.5]),
        ),
        (
            ALIFActivationFunction(spike_fn=MultiSpike, reset_fn=MembraneSubtract()),
            torch.tensor([2.0, 0.0]),
            torch.tensor([0.1, 1.5]),
        ),
        (
            ALIFActivationFunction(spike_fn=MultiSpike, reset_fn=MembraneReset()),
            torch.tensor([2.0, 0.0]),
            torch.tensor([0.0, 1.5]),
        ),
    ],
)
def test_alif_activation_functions(act_fn, output, v_mem):
    state = {
        "v_mem": torch.tensor([0.7, 1.5], requires_grad=True),
        "threshold": torch.tensor([0.3, 2.0]),
    }
    spikes, new_state = act_fn(state)

    assert torch.allclose(spikes, output)
    assert torch.allclose(new_state["v_mem"], v_mem)
    assert torch.allclose(new_state["threshold"], state["threshold"])

    loss = torch.nn.functional.mse_loss(spikes, torch.ones_like(spikes))
    loss.backward()

    assert not (state["v_mem"].grad == 0).all()
    assert not torch.isnan(state["v_mem"].grad).any()
    assert not torch.isinf(state["v_mem"].grad).any()

==> sinabs-0.3.3.dev7.tar.gz: sinabs-0.3.3.dev7/tests/test_activations.py <==
import pytest
import torch
from sinabs.activation import (
    MembraneSubtract,
    MembraneReset,
    MultiSpike,
    SingleSpike,
    SingleExponential,
)


@pytest.mark.parametrize(
    "spike_fn, reset_fn, output, v_mem",
    [
        (
            SingleSpike,
            MembraneSubtract(),
            torch.tensor([1.0, 0.0]),
            torch.tensor([1.5, 0.3]),
        ),
        (
            SingleSpike,
            MembraneReset(),
            torch.tensor([1.0, 0.0]),
            torch.tensor([0.0, 0.3]),
        ),
        (
            MultiSpike,
            MembraneSubtract(),
            torch.tensor([2.0, 0.0]),
            torch.tensor([0.5, 0.3]),
        ),
        (
            MultiSpike,
            MembraneReset(),
            torch.tensor([2.0, 0.0]),
            torch.tensor([0.0, 0.3]),
        ),
    ],
)
def test_activation_functions(spike_fn, reset_fn, output, v_mem):
    state = {"v_mem": torch.tensor([2.5, 0.3], requires_grad=True)}

    spike_threshold = 1.0

    input_tensors = [state[name] for name in spike_fn.required_states]
    spikes = spike_fn.apply(*input_tensors, spike_threshold, SingleExponential())
    new_state = reset_fn(spikes, state, spike_threshold)

    assert torch.allclose(spikes, output)
    assert torch.allclose(new_state["v_mem"], v_mem)

    loss = torch.nn.functional.mse_loss(spikes, torch.ones_like(spikes))
    loss.backward()

    assert not (state["v_mem"].grad == 0).all()
    assert not torch.isnan(state["v_mem"].grad).any()
    assert not torch.isinf(state["v_mem"].grad).any()
